function [wshr, ppvec] = bayesshrink(w,I,sig)

% usage: wshr = bayesshrink(w,I,sig) 
%  or: [wshr ppvec] = bayesshrink(w,I,sig)
% input: w = noisy wavelet coefficients
%        I = measure for prior importance of a coefficient, in particular a
%            measure of the scale on which the coefficient lives, e.g. the
%            support of corresponding wavelet basis functions (up to a
%            constant) or some power of this support, e.g. the square root
%            the algorithm works with the absolute value of this measure.
%      sig = noise deviation in each coefficient (up to a constant)
% output: wshr: shrunk wavelet coefficients
%        ppvec: three vectors of empirical prior probabilities; three estimates
%               the first estimate is a global estimate for the whole scale
%               the second one is a local correction for the basis function
%               support
%               the third one is continuous correction
%               use plot(ppvec(:,1)), plot(ppvec(:,2)), or plot(ppvec(:,3))
%               to visualise. Compare with plot(w) and plot(wshr)

if nargin < 3,
   sig = 1;
end
wsize = size(w);
w = row(w);
sig = row(sig);
I = row(abs(I));
N = length(w);
ppvec = zeros(N,3);
[sortI indI] = sort(abs(I));
% first define `scales' (such that, if w comes from a regularly sampled signal
% this definition coincides with the classical notion of scale
% Isen = I + sentinel
% II contains support magnitudes of first basis function in scale
clear Isen startlevel II
Isen = [sortI Inf];
II(1) = Isen(1);
II(2) = Isen(floor(N/2));
j = 2;
startlevel(1) = 1;
startlevel(2) = floor(N/2);
if abs(Isen(N)-Isen(1))/abs(Isen(1)) < eps, % all I's equal -> single level
   startlevel(2) = N+1;
end
k = startlevel(2);
while k<N-5, % last 5 coefficients must not be taken in too small, separate
             % bins
  j = j+1;
  II(j) = 2*II(j-1);
  % Note that we use factor 2 here. The notion of scale should be defined such
  % that successive scales in a dyadic multiscale analysis differ by a factor
  % of 2. Area is not a good measure, length is better.
  while(Isen(k) < II(j))
     k = k+1;
  end
  II(j) = Isen(k);
  startlevel(j) = k;
end
startlevel(j) = N+1;
II = II(1:length(II)-1);
% startlevel = startlevel(1:length(startlevel)-1)
ww = w(indI)./sig(indI);
J = length(II);
sigest = MAD1(ww)/0.6745; 
ww = ww/sigest;
wshr = w;
pj = zeros(1,J+2);
Imean = zeros(1,J+2); 
for j = 1:J
   wwj = ww(startlevel(j):startlevel(j+1)-1);
   indj = indI(startlevel(j):startlevel(j+1)-1);
   pest = MmLEJS(wwj);
   n = length(wwj);
   pmin = 1/(1 + (n-1)/(2*log(n)) );
   pj(j+1) = max(pest,pmin);
   Imean(j+1) = mean(abs(I(indj)));
end
Imean(1) = abs(I(indI(1)));
pj(1) = pj(2)*Imean(1)/Imean(2);
Imean(J+2) = abs(I(indI(N)));
pj(J+2) = pj(J+1)*Imean(J+2)/Imean(J+1);
if J < 2,
   pp = pj(j+1);
   ppvec = ones(N,3)*pp;
   for i = 1:N,
      vhat(i) = posmedianJS(ww(i),pp);
   end
   wshr = vhat.*sig*sigest;
else for j = 1:J
      clear vhat
      wwj = ww(startlevel(j):startlevel(j+1)-1);
      indj = indI(startlevel(j):startlevel(j+1)-1);
      n = length(wwj);
      for i = 1:n
        Ii = abs(I(indj(i)));
        if Ii>Imean(j+1),
           pp = pj(j+2)*(Ii-Imean(j+1))+pj(j+1)*(Imean(j+2)-Ii);
           pp = pp / (Imean(j+2) - Imean(j+1));
        else
           pp = pj(j+1)*(Ii-Imean(j))+pj(j)*(Imean(j+1)-Ii);
           pp = pp / (Imean(j+1) - Imean(j));
        end
        pp = min(pp,1);
        pp = max(pp,0);
        ppvec(startlevel(j)+i-1,:) = ...
              [pj(j+1) pj(j+1)*abs(I(indj(i)))/Imean(j+1) pp];
        vhat(i) = posmedianJS(wwj(i),pp);
      end
      wshr(indj) = vhat.*sig(indj)*sigest;
   end % for
end % else
wshr = reshape(wshr,wsize);
