
function f = linLScor(x,y,X,Y,F);

% usage: f = linLScor(x,y,X,Y,F);
% corrected least square plane
% X,Y,F are the input data
% (x,y) are the coordinates in which plane is evaluated
% if (x,y) is outside the convex hull of (X,Y), then this approach proceeds
% in 3 steps: (1.) find the line in the x-y-plane fitting the data cloud in a
% total least squares sense (errors in variables). Next (2.) find on this line
% the orthogonal projection of all data points (X,Y), as well as the projection
% of (x,y). Finally (3.), find the least squares line fitting the projected
% data with their corresponding (unchanged) F-values. The evaluation of this
% least squares line is extrapolated (as a constant) into the original (x,y).

N = length(X);
X = column(X);
Y = column(Y);
F = column(F);
if N == 1,
   f = F*ones(size(x));
elseif N == 2,
   vec1 = [column(x)-X(1) column(y)-Y(1)];
   vec2 = [X(2)-X(1) Y(2)-Y(1)];
   r = (vec1*vec2')'./ norm(vec2)^2; 
   r = min(max(r,0),1); % never allow prediction coefficients outside [0,1]
   f = F(1,:) * (1-r) + F(2,:) * r;
else
   if pointinside(x,y,(1:length(X)),X,Y),
      M = length(x);
      x = column(x);
      y = column(y);
      XY = ones(N,3);
      xy = ones(M,3);
      XY(:,2) = X;
      XY(:,3) = Y;
      xy(:,2) = x;
      xy(:,3) = y; 
      A = XY'*XY;
      a = A\(XY'*F);
      f = xy*a;
   else
      % construct a regression line for the (X,Y) data, project the data (X,Y)
      % onto this line, call the resulting points P, then find a regression for
      % (P,F), and finally apply the same extrapolation as for the case N == 2
   % Errors-in-Variables Regression (Total Least Squares) - see:
   % http://mathworld.wolfram.com/LeastSquaresFittingPerpendicularOffsets.html
      [a,b] = linerrinvar(X,Y);
      [Xp,Yp] = projectpointonline(X,Y,a,b);
      R = (Xp-Xp(1)).^2 + (Yp-Yp(1)).^2;
      if isinf(b),
         R = sqrt(R) .* sign(Yp-Yp(1));
      else
         R = sqrt(R) .* sign(Xp-Xp(1));
      end
      R = R - min(R);
      [A,B] = linLS1d(R,F);
      P = ones(N,1)*A + R*B;
      % check this matrix-equation: the first ROW of P has values for
      % Xp(1),Yp(1)
      % this corresponds to the first element of R (column vector)
      % cfr.: the first ROW of F has values for X(1),Y(1)
      % now calculate projection of (x,y) onto the line (Xp,Yp)
      vec1 = [column(x)-Xp(1) column(y)-Yp(1)];
      [dmax, d] = max((Xp-Xp(1)).^2 + (Yp-Yp(1)).^2);
      vec2 = [Xp(d)-Xp(1) Yp(d)-Yp(1)];
      r = (vec1*vec2')'./ norm(vec2)^2;
      r = min(max(r,0),1);
      f = P(1,:) * (1-r) + P(N,:) * r;
   end
end
